"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ElementBuilder = void 0;
const chalk_1 = __importDefault(require("chalk"));
const vars_1 = require("./../libs/vars");
const events_manager_1 = require("./../managers/events-manager");
const element_shadow_1 = require("./element-shadow");
const root_scene_1 = require("./root-scene");
class ElementBuilder {
    scene;
    elem;
    constructor(scene) {
        this.scene = scene;
    }
    log(level) {
        if (this.elem)
            this.elem.log = level;
        return this;
    }
    logger(logger, reused = false) {
        if (this.elem) {
            let thatLogger = reused ? logger : undefined;
            Object.defineProperty(this.elem, 'logger', {
                get() {
                    return thatLogger || (thatLogger = logger?.clone(this.tag));
                }
            });
        }
        return this;
    }
    tag(name) {
        if (this.elem) {
            this.elem.tag = name;
        }
        return this;
    }
    parent(g) {
        if (this.elem) {
            Object.defineProperty(this.elem, 'parent', {
                value: g,
                writable: false
            });
        }
        return this;
    }
    element(ElementClazz, ...args) {
        (0, element_shadow_1.copyElementShadowPrototype)(ElementClazz.prototype);
        this.elem = new ElementClazz(...args);
        const that = this;
        Object.defineProperties(this.elem, {
            scene: {
                get() {
                    return that.scene;
                }
            },
            rootScene: {
                get() {
                    return that.scene instanceof root_scene_1.RootScene ? that.scene : that.scene.rootScene;
                }
            }
        });
        const init = this.elem.init || (() => { });
        this.elem.init = async function (elemProps) {
            events_manager_1.GlobalEvent.emit('element/init');
            await init.call(this, elemProps);
            this.$$markElementNeedReplace();
        };
        const exec = this.elem.exec;
        this.elem.exec = async function (parentResult) {
            try {
                if (this.id) {
                    if (vars_1.Vars.isGlobal(this.id)) {
                        this.rootScene.setVars(this.id, this);
                    }
                    else {
                        this.scene.setVars(this.id, this);
                    }
                }
                if (parentResult) {
                    Object.keys(parentResult)
                        .forEach(key => {
                        const name = 'parent' + key[0].toUpperCase() + key.substring(1);
                        this[name] = parentResult[key];
                    });
                }
                events_manager_1.GlobalEvent.emit('element/exec');
                await this.evalPropsBeforeExec();
                this.logger.setLevel(this.$$getLoggerLevel());
                this.title && this.logger.info(`%s`, this.title);
                this.result = await exec.call(this, parentResult);
            }
            catch (err) {
                this.error = err;
                if (!this.ignoreError)
                    throw this.error;
                this.logger.debug(chalk_1.default.yellow(`⚠️ ${this.error?.message || this.error}`));
            }
            await this.setVarsAfterExec();
            return this.result;
        };
        const dispose = this.elem.dispose || (() => { });
        this.elem.dispose = async function () {
            events_manager_1.GlobalEvent.emit('element/dispose');
            await dispose.call(this);
        };
        return this;
    }
    build() {
        if (!this.elem)
            throw new Error('');
        return this.elem;
    }
}
exports.ElementBuilder = ElementBuilder;
//# sourceMappingURL=element-builder.js.map