"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Group = void 0;
const lodash_clonedeep_1 = __importDefault(require("lodash.clonedeep"));
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const logger_1 = require("../../libs/logger");
const element_shadow_1 = require("../element-shadow");
class Group extends element_shadow_1.ElementShadow {
    $$ignoreEvalProps = ['runs', 'asyncJobs'];
    runs = [];
    asyncJobs = [];
    init(props) {
        if (Array.isArray(props)) {
            this.runs = props;
        }
        else {
            Object.assign(this, props);
        }
        this.runs = this.runs?.filter(e => e) || [];
        this.asyncJobs = [];
    }
    async createNewElement(nameOrClass, ...args) {
        let elem;
        if (typeof nameOrClass === 'string') {
            const name = nameOrClass;
            const ElemClass = await this.rootScene.tagsManager.loadElementClass(name, this.scene);
            elem = this.scene.elementBuilder
                .element(ElemClass, ...args)
                .tag(name)
                .parent(this)
                .logger(this.logger)
                .build();
        }
        else {
            const name = nameOrClass.name || nameOrClass.constructor?.name;
            const ElemClass = nameOrClass;
            elem = this.scene.elementBuilder
                .element(ElemClass, ...args)
                .tag(name)
                .parent(this)
                .logger(this.logger)
                .build();
        }
        return elem;
    }
    async exec(input) {
        const runs = new Array();
        for (const _e of this.runs) {
            const e = typeof _e === 'string' ? { 'echo': { content: _e } } : _e;
            const name = Object.keys(e)[0];
            let props = e[name];
            if (props && typeof props === 'object' && !Array.isArray(props)) {
                let { '<-': inheritKeys, '->': exposeKey, ...eProps } = props;
                if (inheritKeys)
                    eProps = (0, lodash_merge_1.default)({}, this.$$extend(inheritKeys), eProps);
                if (exposeKey)
                    this.$$export(exposeKey, (0, lodash_clonedeep_1.default)(eProps));
                props = eProps;
            }
            if (props?.skip)
                continue;
            const loop = props?.loop;
            if (loop === undefined) {
                if (this.asyncJobs.length && !props?.async) {
                    await Promise.all(this.asyncJobs);
                    this.asyncJobs.splice(0, this.asyncJobs.length);
                }
                const elem = await this.createNewElement(name);
                await elem.init((0, lodash_clonedeep_1.default)(props));
                const isRun = await this.execElement(elem, input);
                isRun && runs.push(elem);
            }
            else {
                let loopCondition = await this.scene.getVars(loop, this);
                if (loopCondition) {
                    if (Array.isArray(loopCondition)) {
                        for (let i = 0; i < loopCondition.length; i++) {
                            const newElem = await this.createNewElement(name);
                            await newElem.init((0, lodash_clonedeep_1.default)(props));
                            newElem.loopKey = i;
                            newElem.loopValue = loopCondition[newElem.loopKey];
                            const isRun = await this.execElement(newElem, input);
                            isRun && runs.push(newElem);
                        }
                    }
                    else if (typeof loopCondition === 'object') {
                        for (const key in loopCondition) {
                            const newElem = await this.createNewElement(name);
                            await newElem.init((0, lodash_clonedeep_1.default)(props));
                            newElem.loopKey = key;
                            newElem.loopValue = loopCondition[newElem.loopKey];
                            const isRun = await this.execElement(newElem, input);
                            isRun && runs.push(newElem);
                        }
                    }
                    else if (loopCondition === true) {
                        while (loopCondition) {
                            const newElem = await this.createNewElement(name);
                            await newElem.init((0, lodash_clonedeep_1.default)(props));
                            newElem.loopValue = loopCondition;
                            const isRun = await this.execElement(newElem, input);
                            isRun && runs.push(newElem);
                            loopCondition = await this.scene.getVars(loop, this);
                        }
                    }
                }
            }
        }
        if (this.asyncJobs.length) {
            await Promise.all(this.asyncJobs);
            this.asyncJobs.splice(0, this.asyncJobs.length);
        }
        return runs;
    }
    async execElement(elem, input) {
        const isContinue = await elem.$$prepare();
        if (!isContinue)
            return false;
        const p = new Promise(async (resolve, reject) => {
            const isAddIndent = this.logger.is(logger_1.LoggerLevel.INFO) && this.parent?.title;
            if (isAddIndent)
                this.logger.addIndent();
            try {
                await elem.exec(input);
                resolve(undefined);
            }
            catch (err) {
                reject(err);
            }
            finally {
                await elem.dispose();
                if (isAddIndent)
                    this.logger.removeIndent();
            }
        });
        if (!elem.async) {
            await p;
        }
        else {
            this.asyncJobs.push(p);
        }
        return true;
    }
}
exports.Group = Group;
//# sourceMappingURL=group.js.map